#
#
# Copyright (c) 1993-2005 Easysoft Ltd. All rights reserved.
#
# $Id: ESXMLODBC.py 51 2005-08-16 12:48:42Z wez $
#
# XML_ODBC interface
# This class is based on the original submission to Easysoft by
# Richard Whitehurst (8-Aug-2005). Thanks Richard.
#
# This is a list of variables that are passed into the class by
# xmlodbc_client.py:
#
#	host               -> -h switch
#	port               -> -p switch
#	DSN                -> -d switch
#	timeout            -> -t switch
#	select_granularity -> -g switch


import socket
import select
import xml.parsers.expat
import sys

class TimeoutError(Exception):
	'''Error Class to raise Timeout Exception.'''
        def __init__(self, value):
                self.value = value
        def __str__(self):
                return repr(self.value)

class ESXMLODBC:
	'''
		Connects to server that services XML requests for data from MS SQL 
		database. Provides send data and read data methods.
	'''

	# These Variables are set as constants but can be changed here.
	_reply_format = "Long"     # Only this format supported
	length = 1024                   # Chunk Size

	def __init__(self):


		try:
			'''Establish tcp socket connection.'''
			self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.socket.connect((self.host,int(self.port)))
		except (socket.gaierror, socket.error, AttributeError), connerror:
			if socket.gaierror == 'true':
				print "socket.gaierror: ", connerror
				print "See usage"
				sys.exit(0)
			elif socket.error == 'true':
				print "socket.error: ", connerror
				print "See usage"
				sys.exit(0)
			else:
				print connerror
				print "See usage"


	def Execute(self,sql):
		self.state = ''
		self.stack = []
		self.rowhash = {}
		self.hasharray = []
		self.rowcount = 0
		self.sendData(sql)
		self.readData()
		if self.state == '00000':
			return True
		else:
			return False


	def sendData(self,sql):
		try:
			'''Send an SQL request in XML format.'''
			eSQL = self._escape(sql)
			self.Request = '''<?xml version="1.0"?>\r
                        <Request>\r
                        <ConnectionString>DSN=%s</ConnectionString>\r
                        <Sql>%s</Sql>\r
                        <Format><%s/></Format>\r
                        </Request>\r''' % (self.DSN,eSQL,self._reply_format)
                        bytecount = self.socket.send(self.Request)
                        return bytecount
		except (AttributeError, socket.error), attriberror:
			print 'ERROR:', attriberror
			print 'Possible connection failure: check port and host'
			print 'See usage'
			sys.exit(0)
	def readData(self):
		try:
			EOM = '</Result>'
			self.XML = ''
			lapsed = 0.0	#Timeout lapse time should be 0.0
			i = []		#Used to check status of i
			TimeoutOff = 0	#Flag to stop timeout checking
			while (i == []):
				'''
				Checks the read status of the socket until it   				changes, if lapse time is > timeout it raises an 				exception if the timeout limit is reached
				'''
				i,o,e = select.select([self.socket],[],[],self.select_granularity)	
				lapsed += self.select_granularity
				if ((lapsed)-self.select_granularity > self.timeout):
					TimeoutOff = 0
					#raise timeout exception
					raise TimeoutError, 'Timeout Reached'
				else:
					TimeoutOff = 1
			while (TimeoutOff == 1) and (self.XML[-10:-1] != EOM):
				'''
				Providing you received an initial reply from the 				server, this loop collects all the data until it 				sees </Result>					
				'''
				self.XML += self.socket.recv(int(self.length))
		except (TimeoutError), error:
			if TimeoutError == 'true':
				print "TimeoutError:", error
				sys.exit(0)
				print "See Usage"
			else:
				print "ERROR: ", error
				print "See usage"

	def closeLink(self):
		self.socket.close()


	def _escape(self,sql):
		'''Convert some of the 5 predefined XML entities'''
		if sql.startswith('<![CDATA['):
			return sql
		esql =  sql.replace('&','&amp;')
		esql = esql.replace('<','&<;')
		esql = esql.replace('>','&>;')
		return esql


	def _parse(self,xml_data):
		'''For internal use. Convert XML to array of hashes.
		Supports only TableTags reply format.
		Empty XML columns are ignored rather than blanked.
		Example of printing array of hashes :-
			XMLODBC.hasharray
				for col in row.keys():
				print col,':', row[col]'''

		def start_element(name, attrs):
			if name == 'Result':
				self.state = attrs['State']
			self.stack.append(name)

		def end_element(name):
			if name[:4] == 'SQL_':
				self.hasharray.append(self.rowhash.copy())
				self.rowhash = {}
				self.rowcount += 1
			self.stack.pop()

		def char_data(data):
			if self.rowhash.has_key(self.stack[-1]):
				self.rowhash[self.stack[-1]] += data
			else:
				self.rowhash[self.stack[-1]] = data

		p = xml.parsers.expat.ParserCreate()
		p.StartElementHandler = start_element
		p.EndElementHandler = end_element
		p.CharacterDataHandler = char_data
		p.returns_unicode = 0	# problem with default of 1
		# Does not like invalid xml such as spaces in element names
		try:
			p.Parse(xml_data,1)
		except xml.parsers.expat.ExpatError,e:
			print 'Failed to parse reply:',e,xml_data

