#!/usr/bin/python
#
# Copyright (c) 1993-2005 Easysoft Ltd. All rights reserved.
#
# $Id: xmlodbc_client.py 55 2005-08-17 08:58:50Z wez $
#
# This is an example python script using the ESXMLODBC class to
# send a SQL request in XML to the Easysoft XML-ODBC Server and
# output the result in XML.
#
# Usage:
#
#	All SWITCHES REQUIRED
#	xmlodbc_client.py 
#	 -h server
#	[-p port] 
#	 -d DSN string
#	[-t timeout]
#	[-g granularity]
#	 -s SQL statement
#
#
# * Minimum Requirements *
#
# xmlodbc_client.py 
#			-h myserver.company.co.uk
#				-> name of machine where XML-ODBC server is 
#				-> running
#                   	-d "test;UID=dbuser;PWD=dbpassword;"
#				-> ODBC connection string
#                       -s "SELECT * FROM mytable"
#				-> the SQL statement you want to issue
# * Optional *
#			-g
#				-> The timeout increments used (granularity)	#		
#
#			-p 8895
#				-> port you want to connect to (default: 8895)
#			-t 60
#				-> timeout after 60 seconds if no reply from    #				   server
#			
#	Note:
#		For an explanation of granularity and timeout, see the README
#
# this causes the XML-ODBC Server on myserver.company.co.uk listening
# on port 8895 to connect to the test data source with a database username
# and password of dbuser/dbpassword, execute the select statement and returns
# the results in XML.
#
# 
#


import string, sys, getopt
from ESXMLODBC import ESXMLODBC

def usage():
	print ""
	print "  All SWITCHES REQUIRED "
	print "  -> excludes: -p -t -g "
	print "  xmlodbc_client.py"
	print "   -h host          -> server name or IP              "
        print "   -d DSN string    -> ODBC connection string         "
        print "   -s SQL statement -> SQL statement to be executed   "
	print "  [-p port]         -> port to connect to             "
	print "  [-t timeout]      -> timeout value (seconds)        "
	print "  [-g granularity]  -> timeout increment value (Seconds)"


sql = ""  #define sql string


# retrieve command line arguments

if len(sys.argv)==1:
    usage()
    sys.exit(0)

args = sys.argv[1:]

try:
	opts, args = getopt.getopt(args, 'h:p:d:l:t:g:e:s:')
except getopt.GetoptError:
	print "invalid command line" 
	usage()
	sys.exit(0)

ESXMLODBC.host = ""
ESXMLODBC.port = ""
ESXMLODBC.DSN = ""
ESXMLODBC.timeout = ""
ESXMLODBC.select_granularity = ""

# process command flags

for opt, val in opts:
	if opt == "-h":
		if val == "":
			usage()
			print "ERROR: Missing or invalid Host name"
			sys.exit(0)
		else:
			ESXMLODBC.host = val 
 	if opt == "-p":
		try:
        		ESXMLODBC.port = int(val)
		except ValueError:
        		usage()
        		print "ERROR: Missing or invalid port e.g. 8895"
			sys.exit(0)
	if opt == "-d":
		if val == "":
			usage()
			print "ERROR: Missing or invalid Connection String"
			sys.exit(0)
		else:
	                ESXMLODBC.DSN = val

	if opt == "-t":
        	if val == "":
        		usage()
        		print "ERROR: Missing or invalid Timeout e.g. 60"
        		sys.exit(0)
		else:
        		try:
                		ESXMLODBC.timeout = int(val)
        		except ValueError:
                		usage()
                		print "ERROR: Missing or invalid Timout e.g. 60"
                		sys.exit(0)

	if opt == "-g":
                if val == "":
                        print "ERROR: Missing or invalid granularity e.g. 0.1"
                        sys.exit(0)
                elif opt == "-g":
                        try:
                                ESXMLODBC.select_granularity = float(val)
                        except ValueError:
                                usage()
                                print "ERROR: Missing or invalid granularity e.g. 0.1"
                                sys.exit(0)

        if opt == "-s":
		if val == "":
			sql = ""
			usage()
			print "ERROR: Missing or invalid SQL e.g. SELECT * FROM sample"
			sys.exit(0)                	
		else:
			sql = val

# If any of these values are "" then they are defaulted

if ESXMLODBC.port == "":
	ESXMLODBC.port = 8895
if ESXMLODBC.timeout == "":
	ESXMLODBC.timeout = 60
if ESXMLODBC.select_granularity == "":
        ESXMLODBC.select_granularity = 0.1

dlink = ESXMLODBC()


if not dlink.Execute(sql):
	print dlink.XML	 
	dlink.closeLink()
